use gtk4 as gtk;

use std::boxed::Box as BoxPtr;
use gtk::{Notebook, Label, Box};
use super::builder_pattern_traits::*;

pub type TabLabel = Label;
pub type TabContent = Box;
pub type TabPage = (BoxPtr<str>, TabContent);

#[derive(Clone)]
pub struct TabsBuilder {
    tabs: Vec<(TabLabel, TabContent)>
}

pub struct Tabs {
    tabs_wrapper: Notebook
}

impl Product<TabsBuilder, Notebook> for Tabs {

    fn builder() -> TabsBuilder {
        TabsBuilder {
            tabs: Vec::new(),
        }
    }

    fn get(self) -> Notebook {
        self.tabs_wrapper
    }

}

impl Builder<Tabs, TabPage, &str> for TabsBuilder{

    fn build(&self, build_param: &str) -> Tabs {
        let tabs_wrapper = Notebook::builder()
            .group_name(build_param)
            .build();

        self.tabs
            .iter()
            .for_each(|(label, content)| {
                tabs_wrapper.append_page(content, Some(label));
            });

        Tabs {
            tabs_wrapper
        }
    }

    fn append_item(mut self, item: TabPage) -> Self {
        self.append_tab_private(&item.0, item.1);

        self
    }

    fn append_items(mut self, items: Vec<TabPage>) -> Self {
        for item in items{
            self.append_tab_private(&item.0, item.1);
        }

        self
    }

}


impl TabsBuilder {

    fn append_tab_private(&mut self, label: &str, page: TabContent) {
        let tab_label = Label::new(Some(label));
        self.tabs.push((tab_label, page));
    }

}